<?php

namespace Drupal\testsuite;

use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Test Suite Services.
 */
class LoadResourceService extends BaseService {

  /**
   * List of directories for the menu.
   *
   * @var array
   */
  protected $menuArray = [
    'custom' => [],
    'contrib' => [],
    'core' => [],
  ];

  /**
   * List of custom tests.
   *
   * @var array
   */
  protected $customTestsArray = [];

  /**
   * List of phpunit tests.
   *
   * @var array
   */
  protected $formReportArray = [];

  /**
   * List of links.
   *
   * @var array
   */
  protected $listFilesArray = [];

  /**
   * Request Stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Absolute path to the resource.
   *
   * @var string
   */
  protected $path;

  /**
   * LoadResourceService constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   */
  public function __construct(RequestStack $requestStack) {
    $this->requestStack = $requestStack;
    $this->path = dirname(dirname(dirname(dirname(__DIR__))));
  }

  /**
   * Gets the array of files to create classes with.
   *
   * @param string $caller
   *   The page type. Menu, form, report etc.
   * @param string $testType
   *   The type of test. Custom or phpunit tests.
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $module
   *   The module name.
   * @param string $directory
   *   The test directory. Unit, Functional etc.
   *
   * @return array
   *   The array of files to create classes with.
   */
  public function getResource($caller, $testType, $area, $module, $directory) {
    switch ($testType) {
      case 'custom':
        $this->getCustomTests('custom', $this->path . '/modules/custom');
        return $this->customTestsArray;

      case 'phpunit':
        if ($caller == 'menu') {
          $this->getDirectories('core', $this->path . '/core/modules');
          $this->getDirectories('contrib', $this->path . '/modules/contrib');
          $this->getDirectories('custom', $this->path . '/modules/custom');
          return $this->menuArray;
        }
        else {
          switch ($area) {
            case 'core';
              $this->getFormReportArray('core', $module, $directory, $this->path . '/core/modules');

              break;

            case 'contrib';
              $this->getFormReportArray('contrib', $module, $directory, $this->path . '/modules/contrib');

              break;

            case 'custom';
              $this->getFormReportArray('custom', $module, $directory, $this->path . '/modules/custom');

              break;
          }
          return $this->formReportArray;
        }
    }
  }

  /**
   * Returns the phpunit statement with a file.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $module
   *   The module name.
   * @param string $directory
   *   The test directory. Unit, Functional etc.
   * @param string $filePath
   *   The absolute path the tests.
   *
   * @return string
   *   The statement to run a phpunit test.
   */
  public function getFileStatement($area, $module, $directory, $filePath) {
    $parts = explode("/", $filePath);
    $file = end($parts).".php";
    // dd($file);
    if ($this->hasWebFolder()) {
      if ($this->isWindows()) {
        switch ($area) {
          case 'core':
            return 'vendor\bin\phpunit --debug -c web\core web\\' . $area . '\modules\\' . $module . '\tests\src\\' . $directory . '\\' . $file;

          case 'contrib':
            return 'vendor\bin\phpunit --debug -c web\core web\modules\contrib\\' . $module . '\tests\src\\' . $directory . '\\' . $file;

          case 'custom':
            return 'vendor\bin\phpunit --debug -c web\core web\modules\custom\\' . $module . '\tests\src\\' . $directory . '\\' . $file;
        }
      }
      else {
        switch ($area) {
          case 'core':
            return 'vendor/bin/phpunit --debug -c web/core web/' . $area . '/modules/' . $module . '/tests/src/' . $directory . '/' . $file;

          case 'contrib':
            return 'vendor/bin/phpunit --debug -c web/core web/modules/contrib/' . $module . '/tests/src/' . $directory . '/' . $file;

          case 'custom':
            return 'vendor/bin/phpunit --debug -c web/core web/modules/custom/' . $module . '/tests/src/' . $directory . '/' . $file;
        }
      }
    }
    else {
      if ($this->isWindows()) {
        switch ($area) {
          case 'core':
            return 'vendor\bin\phpunit --debug -c core ' . $area . '\modules\\' . $module . '\tests\src\\' . $directory . '\\' . $file;

          case 'contrib':
            return 'vendor\bin\phpunit --debug -c core modules\contrib\\' . $module . '\tests\src\\' . $directory . '\\' . $file;

          case 'custom':
            return 'vendor\bin\phpunit --debug -c core modules\custom\\' . $module . '\tests\src\\' . $directory . '\\' . $file;
        }
      }
      else {
        switch ($area) {
          case 'core':
            return 'vendor/bin/phpunit --debug -c core ' . $area . '/modules/' . $module . '/tests/src/' . $directory . '/' . $file;

          case 'contrib':
            return 'vendor/bin/phpunit --debug -c core modules/contrib/' . $module . '/tests/src/' . $directory . '/' . $file;

          case 'custom':
            return 'vendor/bin/phpunit --debug -c core modules/custom/' . $module . '/tests/src/' . $directory . '/' . $file;
        }
      }
    }
  }

  /**
   * Returns the link to a report by file.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $module
   *   The module name.
   * @param string $directory
   *   The test directory. Unit, Functional etc.
   * @param string $statement
   *   The statement used in Phpunit to call tests from the CMD.
   *
   * @return string
   *   The link to a report.
   */
  private function getUrls($area, $module, $directory, $statement = NULL) {
    if ($statement) {
      $parts = explode("/", $statement);
      $file = explode(".", end($parts))[0];
      $link = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost() . "/admin/testsuite/phpunit/file/report/" . $area . "/" . $module . "/" . $directory . "/" .  $file;
    }
    else {
      $link = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost() . "/admin/testsuite/phpunit/file/report/" . $area . "/" . $module . "/" . $directory;
    }

    return $link;
  }

  /**
   * Returns the files listed under each directory.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $module
   *   The module name.
   * @param string $directory
   *   The test directory. Unit, Functional etc.
   *
   * @return array
   *   The list of links to reports by file.
   */
  public function getFiles($area, $module, $directory) {
    $path = "";
    switch ($area) {
      case 'core';
        $path = $this->path . '/core/modules';

        break;

      case 'contrib';
        $path = $this->path . '/modules/contrib';

        break;

      case 'custom';
        $path = $this->path . '/modules/custom';

        break;
    }
    $this->getFilesInDirectories($area, $module, $directory, $path);
    return $this->listFilesArray;
  }

  /**
   * Returns the phpunit statement directory.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $module
   *   The module name.
   * @param string $directory
   *   The test directory. Unit, Functional,  etc.
   *
   * @return string
   *   The statement to run a phpunit test.
   */
  public function getStatement($area, $module, $directory) {
    if ($this->hasWebFolder()) {
      if ($this->isWindows()) {
        switch ($area) {
          case 'core':
            return 'vendor\bin\phpunit --debug -c web\core web\\' . $area . '\modules\\' . $module . '\tests\src\\' . $directory;

          case 'contrib':
            return 'vendor\bin\phpunit --debug -c web\core web\modules\contrib\\' . $module . '\tests\src\\' . $directory;

          case 'custom':
            return 'vendor\bin\phpunit --debug -c web\core web\modules\custom\\' . $module . '\tests\src\\' . $directory;
        }
      }
      else {
        switch ($area) {
          case 'core':
            return 'vendor/bin/phpunit --debug -c web/core web/' . $area . '/modules/' . $module . '/tests/src/' . $directory;

          case 'contrib':
            return 'vendor/bin/phpunit --debug -c web/core web/modules/contrib/' . $module . '/tests/src/' . $directory;

          case 'custom':
            return 'vendor/bin/phpunit --debug -c web/core web/modules/custom/' . $module . '/tests/src/' . $directory;
        }
      }
    }
    else {
      if ($this->isWindows()) {
        switch ($area) {
          case 'core':
            return 'vendor\bin\phpunit --debug -c core ' . $area . '\modules\\' . $module . '\tests\src\\' . $directory;

          case 'contrib':
            return 'vendor\bin\phpunit --debug -c core modules\contrib\\' . $module . '\tests\src\\' . $directory;

          case 'custom':
            return 'vendor\bin\phpunit --debug -c core modules\custom\\' . $module . '\tests\src\\' . $directory;
        }
      }
      else {
        switch ($area) {
          case 'core':
            return 'vendor/bin/phpunit --debug -c core ' . $area . '/modules/' . $module . '/tests/src/' . $directory;

          case 'contrib':
            return 'vendor/bin/phpunit --debug -c core modules/contrib/' . $module . '/tests/src/' . $directory;

          case 'custom':
            return 'vendor/bin/phpunit --debug -c core modules/custom/' . $module . '/tests/src/' . $directory;
        }
      }
    }
  }

  /**
   * Builds the custom tests list.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $path
   *   The absolute path the tests.
   */
  private function getCustomTests($area, $path) {
    if (is_dir($path)) {
      $scan = scandir($path);
      unset($scan[array_search('.', $scan, TRUE)]);
      unset($scan[array_search('..', $scan, TRUE)]);
      foreach ($scan as $module) {
        if (is_dir("$path/$module")) {
          $testsPath = "$path/$module/src/CustomTSTests/";
          if (is_dir($testsPath)) {
            $this->customTestsArray = glob($testsPath . '*.php');
          }
        }
      }
    }
  }

  /**
   * Builds the menus directory list.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $path
   *   The absolute path the tests.
   */
  private function getDirectories($area, $path) {
    if (is_dir($path)) {
      $scan = scandir($path);
      unset($scan[array_search('.', $scan, TRUE)]);
      unset($scan[array_search('..', $scan, TRUE)]);
      foreach ($scan as $module) {
        if (is_dir("$path/$module")) {
          $testsPath = "$path/$module/tests/src/";
          if (is_dir($testsPath)) {
            $testsScan = scandir($testsPath);
            unset($testsScan[array_search('.', $testsScan, TRUE)]);
            unset($testsScan[array_search('..', $testsScan, TRUE)]);
            foreach ($testsScan as $directory) {
              if (is_dir("$testsPath/$directory")) {
                $this->menuArray[$area][$module][] = $directory;
              }
            }
          }
        }
      }
    }
  }

  /**
   * Builds the list of files in the phpunit directories.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $moduleModules
   *   The module name.
   * @param string $moduleDirectory
   *   The test directory. Unit, Functional etc.
   * @param string $path
   *   The absolute path the tests.
   */
  private function getFilesInDirectories($area, $moduleModules, $moduleDirectory, $path) {
    if (is_dir($path)) {
      $scan = scandir($path);
      unset($scan[array_search('.', $scan, TRUE)]);
      unset($scan[array_search('..', $scan, TRUE)]);
      foreach ($scan as $module) {
        if ($moduleModules == $module) {
          if (is_dir("$path/$module")) {
            $testsPath = "$path/$module/tests/src";
            if (is_dir($testsPath)) {
              $testsScan = scandir($testsPath);
              unset($testsScan[array_search('.', $testsScan, TRUE)]);
              unset($testsScan[array_search('..', $testsScan, TRUE)]);
              foreach ($testsScan as $directory) {
                if ($moduleDirectory == $directory) {
                  if (is_dir("$testsPath/$directory")) {
                    $globPath = "$path/$module/tests/src/$directory";
                    $globedFiles = glob($globPath . '/*.php');
                    $this->listFilesArray[] = $this->getUrls($area, $moduleModules, $moduleDirectory);
                    foreach ($globedFiles as $file) {
                      $this->listFilesArray[] = $this->getUrls($area, $moduleModules, $moduleDirectory, $file);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  /**
   * Builds the reports lists.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $moduleModules
   *   The module name.
   * @param string $moduleDirectory
   *   The test directory. Unit, Functional etc.
   * @param string $path
   *   The absolute path the tests.
   */
  private function getFormReportArray($area, $moduleModules, $moduleDirectory, $path) {
    if (is_dir($path)) {
      $scan = scandir($path);
      unset($scan[array_search('.', $scan, TRUE)]);
      unset($scan[array_search('..', $scan, TRUE)]);
      foreach ($scan as $module) {
        if (is_dir("$path/$module")) {
          $testsPath = "$path/$module/tests/src/";
          if (is_dir($testsPath)) {
            $testsScan = scandir($testsPath);
            unset($testsScan[array_search('.', $testsScan, TRUE)]);
            unset($testsScan[array_search('..', $testsScan, TRUE)]);
            foreach ($testsScan as $directory) {
              if ($moduleDirectory == $directory) {
                if (is_dir("$testsPath/$directory") && !isset($this->formReportArray[$module])) {
                  if ($moduleModules == 'all') {
                    $this->formReportArray[$module] = $this->getStatement($area, $module, $directory);
                  }
                  else {
                    if ($moduleModules == $module) {
                      $this->formReportArray[$module] = $this->getStatement($area, $module, $directory);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }

}
